"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const blockweave_1 = __importDefault(require("blockweave"));
const promise_pool_1 = require("@supercharge/promise-pool");
const transaction_1 = require("./utils/transaction");
const tags_1 = require("./utils/tags");
const utils_1 = require("./utils/utils");
class ArlocalSweets {
    _blockweave;
    _wallet;
    _mainnet = new blockweave_1.default();
    constructor(blockweave, wallet) {
        this._blockweave = blockweave;
        this._wallet = wallet;
        // Support chaining of methods
        return this;
    }
    async _validateNetwork() {
        const { network } = await this._blockweave.network.getInfo();
        if (!network.toLowerCase().includes("arlocal")) {
            throw new Error(`${network} not supported, use arlocal`);
        }
    }
    /**
     *
     * @returns True if network is valid, and vice versa
     */
    async isTestNetwork() {
        try {
            await this._validateNetwork();
        }
        catch (e) {
            return false;
        }
        return true;
    }
    /**
     *
     * @param amount Amount in winston
     * @returns Amount funded in winston
     */
    async fundWallet(amount) {
        // Validate blockweave
        await this._validateNetwork();
        const address = await this._blockweave.wallets.jwkToAddress(this._wallet);
        const { data } = await this._blockweave.api.get(`/mint/${address}/${amount}`);
        return data;
    }
    /**
     *
     * @param blocks Amount of blocks to mine
     * @returns Amount of blocks mined
     */
    async mine(blocks) {
        // Validate blockweave
        await this._validateNetwork();
        blocks = blocks || 1;
        await this._blockweave.api.get(`/mine/${blocks}`);
        return blocks;
    }
    /**
     *
     * @param txid Mainnet transaction id
     * @returns Arlocal transaction id
     */
    async copyTransaction(txid) {
        // Validate blockweave
        await this._validateNetwork();
        let data = "";
        const { data: resp } = await this._mainnet.api.get(`/tx/${txid}/status`);
        if (typeof resp === "string") {
            throw new Error(`Transaction returned status of: ${resp}`);
        }
        try {
            ({ data } = await this._mainnet.api.get(`/${txid}`));
            if (typeof data === "object") {
                data = JSON.stringify(data);
            }
        }
        catch (e) { }
        const tx = await this._mainnet.transactions.get(txid);
        const id = await (0, transaction_1.copyTx)(tx, this._blockweave, this._wallet, data);
        await this.mine();
        return id;
    }
    /**
     *
     * @param txid Mainnet transaction id
     * @returns Arlocal transaction id
     */
    async cloneTransaction(txid) {
        // Validate blockweave
        await this._validateNetwork();
        let data = "";
        const { data: resp } = await this._mainnet.api.get(`/tx/${txid}/status`);
        if (typeof resp === "string") {
            throw new Error(`Transaction returned status of: ${resp}`);
        }
        try {
            ({ data } = await this._mainnet.api.get(`/${txid}`));
            if (typeof data === "object") {
                data = JSON.stringify(data);
            }
        }
        catch (e) { }
        const tx = await this._mainnet.transactions.get(txid);
        const id = await (0, transaction_1.cloneTx)(tx, this._blockweave, this._wallet, data);
        await this.mine();
        return id;
    }
    /**
     *
     * @param txid Mainnet contract initial state transaction ID
     * @returns Arlocal contract initial state transaction ID
     */
    async copyContract(txid) {
        // Validate blockweave
        await this._validateNetwork();
        let data = "";
        // validate tx
        const { data: resp } = await this._mainnet.api.get(`/tx/${txid}/status`);
        if (typeof resp === "string") {
            throw new Error(`Transaction returned status of: ${resp}`);
        }
        // get state tx meta_data
        const tx = await this._mainnet.transactions.get(txid);
        // validate tx type
        const value = (0, tags_1.searchForTag)(tx, "App-Name");
        if (value !== "SmartWeaveContract") {
            throw new Error("Transaction is not a SmartWeave contract");
        }
        // get contract src
        const src = (0, tags_1.searchForTag)(tx, "Contract-Src");
        if (!src) {
            throw new Error("Contract-Src not found in contract state tags");
        }
        // copy the contract src
        const nsrc = await this.copyTransaction(src);
        // get the state tx data
        try {
            ({ data } = await this._mainnet.api.get(`/${txid}`));
            // fix json parsing by axios
            if (typeof data !== "string") {
                data = JSON.stringify(data);
            }
        }
        catch (e) { }
        // copy the state tx
        const eTags = [
            {
                name: "Contract-Src",
                value: nsrc,
            },
        ];
        const id = await (0, transaction_1.copyTx)(tx, this._blockweave, this._wallet, data, eTags);
        await this.mine();
        return id;
    }
    /**
     *
     * @param txid mainnet manifest ID
     * @returns Testnet manifest ID
     */
    async copyManifestTransaction(txid) {
        // Validate blockweave
        await this._validateNetwork();
        // validate tx
        const { data: resp } = await this._mainnet.api.get(`/tx/${txid}/status`);
        if (typeof resp === "string") {
            throw new Error(`Transaction returned status of: ${resp}`);
        }
        // get state tx meta_data
        const tx = await this._mainnet.transactions.get(txid);
        // validate tx type
        const value = (0, tags_1.searchForTag)(tx, "Content-Type");
        if (value !== "application/x.arweave-manifest+json") {
            throw new Error("Transaction is not an arweave manifest");
        }
        // get mainfest
        const { data: b64manifest } = await this._mainnet.api.get(`/tx/${txid}/data`);
        // parse manifest
        const manifest = JSON.parse((0, utils_1.atob)(b64manifest));
        // get the manifest paths and deploy each paths
        const { results, errors } = await promise_pool_1.PromisePool.for(Object.keys(manifest.paths))
            .withConcurrency(5)
            .process(async (path) => {
            const oldPathId = manifest.paths[path].id;
            const newPathId = await this.copyTransaction(oldPathId);
            return { path, id: newPathId };
        });
        // rebuild manifest
        if (errors.length) {
            console.error(errors);
        }
        const $paths = {};
        results.forEach(({ path, id }) => ($paths[path] = { id }));
        const $manifest = {
            ...manifest,
            paths: $paths,
        };
        // deploy manifest
        const ntx = await this._blockweave.createTransaction({
            data: JSON.stringify($manifest),
        }, this._wallet);
        const tags = tx.get("tags");
        tags.forEach((tag) => {
            ntx.addTag(tag.get("name", { decode: true, string: true }), tag.get("value", { decode: true, string: true }));
        });
        await this._blockweave.transactions.sign(ntx, this._wallet);
        await this._blockweave.transactions.post(ntx);
        await this.mine();
        // return manifest txid
        return ntx.id;
    }
}
module.exports = ArlocalSweets;
